import { Component, Input, OnInit } from '@angular/core';
import { Passenger } from '../../interfaces/passenger.interface';
import { TicketType } from '../../interfaces/ticket-type.interface';
import { FlightClass } from '../../interfaces/flight-class.interface';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { PassengerService } from '../../services/passenger.service';
import { TicketTypeService } from '../../services/ticket-type.service';
import { TicketClassService } from '../../services/ticket-class.service';
import { DropdownElement } from 'src/app/shared/interfaces/dropdown-element-interface';
import { FormControl, FormGroup } from '@angular/forms';

@Component({
  selector: 'agent-base-information',
  templateUrl: './base-information.component.html',
  styleUrls: ['./base-information.component.css']
})
export class BaseInformationComponent {

  public passengerModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'name',
      title: 'Nombre'
    },
    {
      name: 'secondName',
      title: 'Segundo Nombre'
    },
    {
      name: 'lastname',
      title: 'Apellido'
    },
    {
      name: 'email',
      title: 'Correo'
    },
    {
      name: 'phone',
      title: 'Teléfono'
    }
  ];

  @Input()
  public group: FormGroup = new FormGroup({
    passenger: new FormControl<string | null>(null),
    ticketType: new FormControl<string | null>(null),
    flightClass: new FormControl<string | null>(null),
  });

  constructor() { }

  getControl(name: string): FormControl<string | null> {
    return this.group.get(name) as FormControl<string | null>;
  }

  ngOnInit(): void {
  }

  @Input()
  public passengers: Passenger[] = [];

  @Input()
  public ticketTypes: DropdownElement[] = [];

  @Input()
  public flightClass: DropdownElement[] = [];

  onSelectedPassenger(passenger: Passenger): void {
    this.group.get('passenger')?.setValue(passenger.id);
  }

  getPassengerName(id: string | null): string | null {
    if (id) {
      const passenger = this.passengers.find(passenger => passenger.id === Number.parseInt(id));
      return passenger ? `${passenger.name} ${passenger.lastname}` : null;
    } return null;
  }
}
