import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { AirportsService } from '../../services/airports.service';
import { Airport } from '../../interfaces/airport.interface';
import { FlightService } from 'src/app/supervisor/services/flight.service';
import { FormControl, FormGroup } from '@angular/forms';
import { Flight } from 'src/app/supervisor/interfaces/flight.interface';

@Component({
  selector: 'agent-flight-information',
  templateUrl: './flight-information.component.html',
  styleUrls: ['./flight-information.component.css']
})
export class FlightInformationComponent implements OnInit {

  @Input()
  public flightModel: TableModel[] = [];

  public airportModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'name',
      title: 'Nombre'
    },
    {
      name: 'location',
      title: 'Ubicacion'
    },
    {
      name: 'code',
      title: 'Codigo'
    }
  ];

  public flightResults: Flight[] = [];

  @Input()
  public group: FormGroup = new FormGroup({
    airportOrigin: new FormControl<string | null>(null),
    airportDestination: new FormControl<string | null>(null),
    date: new FormControl<string | null>(null)
  });

  getControl(name: string): FormControl<string | null> {
    return this.group.get(name) as FormControl<string | null>;
  }

  public airports: Airport[] = [];


  constructor(
    private airpotsService: AirportsService,
    private flightService: FlightService) { }

  ngOnInit(): void {
    this.airpotsService.getAll()
      .subscribe(airports => this.airports = airports);

    this.flightSelected ? this.flightResults.push(this.flightSelected) : [];
  }

  onSelectAirportOrigin(airport: Airport): void {
    this.getControl('airportOrigin').setValue(airport.name);
  }

  onSelectAirportDestination(airport: Airport): void {
    this.getControl('airportDestination').setValue(airport.name);
  }

  onSearchFlights(): void {
    this.group.markAllAsTouched();
    if (this.group.valid) {
      this.flightService.searchFlight(this.group.value.airportOrigin, this.group.value.airportDestination, this.group.value.date)
        .subscribe(flights => {
          this.flightResults = flights;
        });
    }
  }

  @Output()
  public onSelectFlight: EventEmitter<Flight> = new EventEmitter<Flight>();

  @Input()
  public flightSelected: Flight | null = null;

  onEventSelectFlight(flight: Flight): void {
    this.flightSelected = flight;
    this.onSelectFlight.emit(flight);
  }
}
