import { Component, EventEmitter, Input, Output } from '@angular/core';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { CostFlight } from 'src/app/supervisor/interfaces/cost-flight.interface';
import { Flight } from 'src/app/supervisor/interfaces/flight.interface';

@Component({
  selector: 'agent-summary-information',
  templateUrl: './summary-information.component.html',
  styleUrls: ['./summary-information.component.css']
})
export class SummaryInformationComponent {

  public scaleModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'airportOrigin',
      title: 'Origen'
    },
    {
      name: 'airportDestination',
      title: 'Destino'
    },
    {
      name: 'dateStart',
      title: 'Fecha de salida'
    },
    {
      name: 'dateEnd',
      title: 'Fecha de llegada'
    }
  ]

  @Input()
  public passengerName: string = '';

  @Input()
  public ticketTypeName: string = '';

  @Input()
  public classTypeName: string = '';

  @Input()
  public flightSelected: Flight | null = null;

  @Output()
  public onChangeStep: EventEmitter<number> = new EventEmitter<number>();


  get flighType(): string {
    return this.flightSelected ? this.flightSelected.scales.length > 1 ? 'Con escalas' : 'Directo' : '';
  }

  get totalCost(): number | null {
    const costs = this.flightSelected?.costs;
    const date: Date = new Date(this.flightSelected?.dateStart as string);
    const cost: CostFlight | null = null;

    if (costs) {
      for (let i = 0; i < costs.length; i++) {
        const dateStart: Date = new Date(costs[i].dateStart);
        const dateEnd: Date = new Date(costs[i].dateEnd);

        if (date >= dateStart && date <= dateEnd) {
          return costs[i].cost;
        }
      }
    } return null;
  }

  public changeStep(step: number): void {
    this.onChangeStep.emit(step);
  }
}
