import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Reservation } from '../interfaces/reservation.interface';
import { Observable, tap } from 'rxjs';

@Injectable({providedIn: 'root'})
export class ReservationsService {
  constructor(private http: HttpClient) { }

  URL_BASE: string = 'http://localhost:3000/reservations';

  getAll(): Observable<Reservation[]> {
    return this.http.get<Reservation[]>(this.URL_BASE);
  }

  getById(id: number): Observable<Reservation> {
    return this.http.get<Reservation>(`${this.URL_BASE}/${id}`);
  }

  getByPassengerId(id: number): Observable<Reservation[]> {
    return this.http.get<Reservation[]>(`${this.URL_BASE}?passengerId=${id}`);
  }

  save(reservation: Reservation): Observable<Reservation> {
    return this.http.post<Reservation>(this.URL_BASE, reservation);
  }

  edit(id: number, reservation: Reservation): Observable<Reservation> {
    return this.http.put<Reservation>(`${this.URL_BASE}/${id}`, reservation);
  }

  delete(id: number): Observable<Reservation> {
    return this.http.delete<Reservation>(`${this.URL_BASE}/${id}`);
  }
}
