import { Component, OnInit } from '@angular/core';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { ReservationDetail } from '../../interfaces/reservation-detail.interface';
import { TableAction } from 'src/app/shared/interfaces/table-action.interface';
import { Router } from '@angular/router';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { ReservationsService } from 'src/app/agent/services/reservations.service';
import { FlightService } from 'src/app/supervisor/services/flight.service';
import { Reservation } from 'src/app/agent/interfaces/reservation.interface';

@Component({
  selector: 'passenger-my-reservations',
  templateUrl: './my-reservations.component.html',
  styleUrls: ['./my-reservations.component.css']
})
export class MyReservationsComponent implements OnInit {

  public actions: TableAction[] = [
    {
      title: 'Editar',
      icon: 'bi-pencil-square',
      action: (item) => { console.log(item); }
    },
    {
      title: 'Eliminar',
      icon: 'bi-trash',
      action: (item) => { console.log(item); }
    },
    {
      title: "Pagar",
      icon: "bi-cash",
      action: (item) => { this.onTogleDialogPay(); this.reservationSelected = item; }
    }
  ];

  public reservationSelected: ReservationDetail | null = null;
  public reservation: Reservation | null = null;

  public tableModel: TableModel[] = [
    {
      name: 'id',
      title: '#',
    },
    {
      name: 'origin',
      title: 'Origen',
    },
    {
      name: 'destination',
      title: 'Destino',
    },
    {
      name: 'date',
      title: 'Fecha',
    },
    {
      name: 'status',
      title: 'Estado',
    },
    {
      name: 'cost',
      title: 'Costo',
    }
  ]
  public tableData: ReservationDetail[] = [];

  cardControl = new FormControl<string>('', [Validators.required]);
  expirationControl = new FormControl<string>('', [Validators.required]);
  cvvControl = new FormControl<string>('', [Validators.required]);
  ownerControl = new FormControl<string>('', [Validators.required]);

  formGroup = new FormGroup({
    card: this.cardControl,
    expiration: this.expirationControl,
    cvv: this.cvvControl,
    owner: this.ownerControl,
  });

  onSave(): void {
    if (this.formGroup.valid && this.reservation && this.reservationSelected) {
      this.reservation.statusCode = 'PROCESANDO_PAGO';
      this.reservationSelected.status = 'PROCESANDO_PAGO';
      this.reservationService.edit(this.reservationSelected!.id, this.reservation)
        .subscribe(() => {
          this.formGroup.reset();
          this.onTogleDialogPay();
        });
    }
  }

  public openDialogPay: boolean = false;

  constructor(private router: Router,
    private reservationService: ReservationsService,
    private flightService: FlightService) {}

  ngOnInit(): void {
    this.reservationService.getByPassengerId(4)
      .subscribe(response => {
        response.forEach(reservation => {
          this.reservation = reservation;
          this.flightService.getById(reservation.flightId)
            .subscribe((flight) => {
              const reservationDetail: ReservationDetail = {
                id: reservation.id!,
                origin: flight.airportOrigin,
                destination: flight.airportDestination,
                date: reservation.dateReservationMade,
                status: reservation.statusCode,
                cost: flight.costs[0].cost
              }
              this.tableData.push(reservationDetail);
            })
        })
      })
  }

  onClickItem(item: ReservationDetail): void {
    this.router.navigate(['/passenger/reservations', item.id]);
  }

  onTogleDialogPay(): void {
    this.openDialogPay = !this.openDialogPay;
  }
}
