import { Component } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { SecurityService } from '../../services/security.service';
import { SecurityCredentials } from '../../models/security-credentials.interface';
import { SecurityResponse } from '../../models/security-response.interface';
import { Router } from '@angular/router';
import { UserLogged } from '../../models/user-logged.interface';
import { SupervisorAuthorize } from '../../models/supervisor-authorize.interface';
import { AgentAuthorize } from '../../models/agent-authorize.interface';
import { PassengerAuthorize } from '../../models/passenger-authorize.interface';

@Component({
  selector: 'app-sign-in',
  templateUrl: './sign-in.component.html',
  styleUrls: ['./sign-in.component.css']
})
export class SignInComponent {

  public lenguage: string = 'es';
  public lenguages: any[] = [
    {
      value: 'es',
      text: 'Español'
    },
    {
      value: 'en',
      text: 'English'
    }
  ];

  public text: any = {
    es: {
      email: 'Usuario o email',
      password: 'Contraseña',
      button: 'Iniciar sesión'
    },
    en: {
      email: 'Username or email',
      password: 'Password',
      button: 'Sign in'
    }
  }

  constructor(private securityService: SecurityService,
    private router: Router) {}

  public signInForm: FormGroup = new FormGroup({
    email: new FormControl<string | null>(null, [Validators.required, Validators.minLength(3)]),
    password: new FormControl<string | null>(null, [Validators.required, Validators.minLength(3)])
  });

  public showError: boolean = false;
  public securityResponse: SecurityResponse | null = null;

  changeLenguage(lenguage: string) {
    this.lenguage = lenguage;
  }

  onSignIn(): void {
    this.signInForm.markAllAsTouched();
    if (this.signInForm.valid) {
      const credentials: SecurityCredentials = this.signInForm.value;
      this.securityService.login(credentials)
        .subscribe(securityResponse => {
          this.redirectAndSaveLocalStorage(securityResponse);
        });
    } else this.showError = true;

  }

  redirectAndSaveLocalStorage(securityResponse: SecurityResponse): void {
    switch(securityResponse.user.role) {
      case 'supervisor':
        this.securityService.getSupervisorByUserId(securityResponse.user.id)
          .subscribe(supervisors => {
            this.supervisorSaveLocalStorage(supervisors[0], securityResponse);
            this.router.navigate(['/supervisors']);
          });
        break;
      case 'agent':
        this.securityService.getAgentByUserId(securityResponse.user.id)
          .subscribe(agents => {
            this.agentSaveLocalStorage(agents[0], securityResponse);
            this.router.navigate(['/agents']);
          });
        break;
      case 'passenger':
        this.securityService.getPassengerByUserId(securityResponse.user.id)
          .subscribe(passengers => {
            this.passengerSaveLocalStorage(passengers[0], securityResponse);
            this.router.navigate(['/passengers']);
          });
        break;
    }
  }

  supervisorSaveLocalStorage(supervisor: SupervisorAuthorize, securityResponse: SecurityResponse): void {
    const userLogged: UserLogged = {
      id: supervisor.id,
      name: supervisor.name,
      lastname: supervisor.lastname,
      email: supervisor.user.email,
      role: supervisor.user.role,
      token: securityResponse.accessToken
    }
    localStorage.setItem('userLogged', JSON.stringify(userLogged));
  }

  agentSaveLocalStorage(agent: AgentAuthorize, securityResponse: SecurityResponse): void {
    const userLogged: UserLogged = {
      id: agent.id,
      name: agent.name,
      lastname: agent.lastname,
      email: agent.user.email,
      role: agent.user.role,
      token: securityResponse.accessToken
    }
    localStorage.setItem('userLogged', JSON.stringify(userLogged));
  }

  passengerSaveLocalStorage(passenger: PassengerAuthorize, securityResponse: SecurityResponse): void {
    const userLogged: UserLogged = {
      id: passenger.id,
      name: passenger.name,
      lastname: passenger.lastname,
      email: passenger.user.email,
      role: passenger.user.role,
      token: securityResponse.accessToken
    }
    localStorage.setItem('userLogged', JSON.stringify(userLogged));
  }
}
