
import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { SecurityCredentials } from '../models/security-credentials.interface';
import { SecurityResponse } from '../models/security-response.interface';
import { Observable, of, tap } from 'rxjs';
import { SupervisorAuthorize } from '../models/supervisor-authorize.interface';
import { Passenger } from 'src/app/agent/interfaces/passenger.interface';
import { PassengerAuthorize } from '../models/passenger-authorize.interface';
import { AgentAuthorize } from '../models/agent-authorize.interface';
import { UserLogged } from '../models/user-logged.interface';
import { Router } from '@angular/router';

@Injectable({providedIn: 'root'})
export class SecurityService {
  constructor(private http: HttpClient, private router: Router) { }

  URL_BASE: string = 'http://localhost:3000/';

  get currentUser(): UserLogged | null {
    const userLogged: UserLogged | null = JSON.parse(localStorage.getItem('userLogged') || 'null');
    return userLogged;
  }

  login(securityCredentials: SecurityCredentials): Observable<SecurityResponse> {
    return this.http.post<SecurityResponse>(this.URL_BASE + 'signin', securityCredentials);
  }

  loguot(): void {
    localStorage.removeItem('userLogged');
    this.router.navigate(['/sign-in']);
  }

  getSupervisorByUserId(userId: number): Observable<SupervisorAuthorize[]> {
    return this.http.get<SupervisorAuthorize[]>(this.URL_BASE + 'supervisors?userId=' + userId + '&_expand=user');
  }

  getPassengerByUserId(userId: number): Observable<PassengerAuthorize[]> {
    return this.http.get<PassengerAuthorize[]>(this.URL_BASE + 'passengers?userId=' + userId + '&_expand=user');
  }

  getAgentByUserId(userId: number): Observable<AgentAuthorize[]> {
    return this.http.get<AgentAuthorize[]>(this.URL_BASE + 'agents?userId=' + userId + '&_expand=user');
  }
}
