import { Component, Input, OnInit } from '@angular/core';
import { FormControl, Validators } from '@angular/forms';

@Component({
  selector: 'shared-input-text',
  templateUrl: './input-text.component.html',
  styleUrls: ['./input-text.component.css']
})
export class InputTextComponent implements OnInit {

  @Input()
  public label: string = '';

  @Input()
  public name: string = '';

  @Input()
  public type: string = 'text';

  @Input()
  public maxLength?: number;

  @Input()
  public minLength?: number;

  @Input()
  public control?: FormControl<string | null>;

  ngOnInit(): void {
    if (!this.maxLength) this.maxLength = 999;
    if (!this.minLength) this.minLength = 999;
  }

  isInputValid(): boolean | null {
    return this.control? this.control?.errors && this.control?.touched : null;
  }

  getFieldError(): string | null {
    const errors = this.control?.errors || {};

    for (const key of Object.keys(errors)) {
      switch (key) {
        case 'required':
          return 'Este campo es requerido';
        case 'email':
          return 'El email no es valido';
        case 'minlength':
          return `El minimo de caracteres es ${errors[key].requiredLength}`;
        case 'maxlength':
          return `El maximo de caracteres es ${errors[key].requiredLength}`;
      }
    }
    return null;
  }
}
