import { Component, OnInit } from '@angular/core';
import { PanelItem } from '../../interfaces/panel-item.interface';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { AirportsService } from 'src/app/agent/services/airports.service';
import { Airline } from '../../interfaces/airline.interface';
import { AirlineService } from '../../services/airline.service';
import { ScaleFlight } from '../../interfaces/scale.interface';
import { CostFlight } from '../../interfaces/cost-flight.interface';
import { FormControl, FormGroup, ValidatorFn, Validators } from '@angular/forms';
import { DropdownElement } from 'src/app/shared/interfaces/dropdown-element-interface';
import { AircraftService } from '../../services/aircraft.service';
import { FlightService } from '../../services/flight.service';
import { Flight } from '../../interfaces/flight.interface';
import { ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';

@Component({
  selector: 'app-flight-add',
  templateUrl: './flight-add.component.html',
  styleUrls: ['./flight-add.component.css']
})
export class FlightAddComponent implements OnInit {

  public airlineModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'name',
      title: 'Nombre'
    },
    {
      name: 'code',
      title: 'Código'
    }
  ]

  public scaleModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'airportOrigin',
      title: 'Origen'
    },
    {
      name: 'airportDestination',
      title: 'Destino'
    },
    {
      name: 'dateStart',
      title: 'Fecha Inicio'
    },
    {
      name: 'dateEnd',
      title: 'Fecha Fin'
    }
  ]

  public costModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'dateStart',
      title: 'Fecha Inicio'
    },
    {
      name: 'dateEnd',
      title: 'Fecha Fin'
    },
    {
      name: 'aircraftType',
      title: 'Tipo de Avión'
    },
    {
      name: 'cost',
      title: 'Costo'
    }
  ]

  public panelMenu: PanelItem[] = [
    {
      name: 'scales',
      title: 'Escalas',
      isActive: true,
      isVisible: true
    },
    {
      name: 'costs',
      title: 'Costos',
      isActive: false,
      isVisible: true
    }
  ];

  public panelMenuActive: PanelItem = this.panelMenu[0];

  // Info
  public scales: ScaleFlight[] = [];
  public costs: CostFlight[] = [];

  public openDialogScale: boolean = false;
  public openDialogCost: boolean = false;

  public airlines: Airline[] = [];
  public airports: DropdownElement[] = [];
  public aircrafts: DropdownElement[] = [];


  public airlineValue: string = '';

  public airlineIdControl: FormControl = new FormControl<number | null>(null, [Validators.required]);
  public aircraftTypeIdControl: FormControl = new FormControl<number | null>(null, [Validators.required]);

  public flightBaseConfig: FormGroup = new FormGroup({
    airlineId: this.airlineIdControl,
    aircraftTypeId: this.aircraftTypeIdControl,
  });

  public airportIdOriginControl: FormControl = new FormControl<number | null>(null, [Validators.required]);
  public airportIdDestinationControl: FormControl = new FormControl<number | null>(null, [Validators.required]);
  public dateStartControl: FormControl = new FormControl<string | null>(null, [Validators.required]);
  public dateEndControl: FormControl = new FormControl<string | null>(null, [Validators.required]);

  public scalesForm: FormGroup = new FormGroup({
    airportIdOrigin: this.airportIdOriginControl,
    airportIdDestination: this.airportIdDestinationControl,
    dateStart: this.dateStartControl,
    dateEnd: this.dateEndControl,
  });

  public dateStartCostControl: FormControl = new FormControl<string | null>(null, [Validators.required]);
  public dateEndCostControl: FormControl = new FormControl<string | null>(null, [Validators.required]);
  public costControl: FormControl = new FormControl<number | null>(null, [Validators.required]);
  public aircraftIdControl: FormControl = new FormControl<number | null>(null, [Validators.required]);

  public costsForm: FormGroup = new FormGroup({
    dateStart: this.dateStartCostControl,
    dateEnd: this.dateEndCostControl,
    cost: this.costControl,
    aircraftId: this.aircraftIdControl,
  });

  constructor(private airportsService: AirportsService,
    private airlinesService: AirlineService,
    private aircraftService: AircraftService,
    private flightService: FlightService,
    private activatedRoute: ActivatedRoute,
    private location: Location) {
    this.addCustomValidatorsInScaleForm();
    this.addCustomValidatorsInCostForm();
  }

  ngOnInit(): void {
    this.getAllData();

    this.activatedRoute.params
      .subscribe(params => {
        const id = params['id'];
        if (id) {
          this.flightService.getById(id)
            .subscribe(flight => {
              this.airlineValue = flight.airline;
              this.airlineIdControl.setValue(flight.airlineId);
              this.aircraftTypeIdControl.setValue(flight.aircraftTypeId);
              this.scales = flight.scales;
              this.costs = flight.costs;
            });
        }
      });
  }

  getAllData(): void {
    this.airlinesService.getAll()
    .subscribe(airlines => this.airlines = airlines);

    this.airportsService.getAll()
      .subscribe(airports => {
        const airportsDropdown: DropdownElement[] = [];
        airports.map(airport => {
          airportsDropdown.push({
            text: airport.name,
            value: airport.id.toString()
          });
        });
        this.airports = airportsDropdown;
      });

    this.aircraftService.getAll()
      .subscribe(response => {
        const aircrafts: DropdownElement[] = [];
        response.map(aircraft => {
          aircrafts.push({
            text: aircraft.name,
            value: aircraft.id.toString()
          });
        });
        this.aircrafts = aircrafts;
      })
  }

  onTogleDialogScale(): void {
    this.openDialogScale = !this.openDialogScale;
    this.scalesForm.reset();
  }

  onTogleDialogCost(): void {
    this.openDialogCost = !this.openDialogCost;
    this.costsForm.reset();
  }

  onSelectAirline(airline: Airline): void {
    this.airlineValue = airline.name;
    this.airlineIdControl.setValue(airline.id);
  }

  onchangeItemActive(item: PanelItem): void {
    this.panelMenuActive = item;
  }

  // Getters

  isInvalidFlightConfiguration(): boolean {
    return this.flightBaseConfig.invalid || this.scales.length == 0 || this.costs.length == 0;
  }

  // Custom Validators

  addCustomValidatorsInScaleForm(): void {
    this.dateEndControl.addValidators(this.dateValidator());
    this.dateStartControl.addValidators(this.dateValidator());
    this.airportIdOriginControl.addValidators(this.airportValidator());
    this.airportIdDestinationControl.addValidators(this.airportValidator());

    this.dateStartControl.valueChanges.subscribe(() => {
      this.dateEndControl.updateValueAndValidity();
    });
    this.dateEndControl.valueChanges.subscribe(() => {
      this.dateStartControl.updateValueAndValidity();
    });
    this.airportIdOriginControl.valueChanges.subscribe(() => {
      this.airportIdDestinationControl.updateValueAndValidity();
    });
    this.airportIdDestinationControl.valueChanges.subscribe(() => {
      this.airportIdOriginControl.updateValueAndValidity();
    });
  }

  addCustomValidatorsInCostForm(): void {
    this.dateEndCostControl.addValidators(this.dateCostValidator());
    this.dateStartCostControl.addValidators(this.dateCostValidator());

    this.dateStartCostControl.valueChanges.subscribe(() => {
      this.dateEndCostControl.updateValueAndValidity();
    });
    this.dateEndCostControl.valueChanges.subscribe(() => {
      this.dateStartCostControl.updateValueAndValidity();
    });
  }

  dateValidator(): ValidatorFn {
    return () => {
      const dateStart = this.dateStartControl.value;
      const dateEnd = this.dateEndControl.value;

      if(!dateStart || !dateEnd) return { isValid: false };
      if(dateStart === dateEnd) return { isValid: false };
      if (new Date(dateStart) > new Date(dateEnd)) return { isValid: false };

      return null;
    };
  }

  airportValidator(): ValidatorFn {
    return () => {
      const airportOrigin = this.airportIdOriginControl.value;
      const airportDestination = this.airportIdDestinationControl.value;

      if(!airportOrigin || !airportDestination) return { isValid: false };

      if(airportOrigin === airportDestination) return { isValid:false };

      return null;
    };
  }

  dateCostValidator(): ValidatorFn {
    return () => {
      const dateStart = this.dateStartCostControl.value;
      const dateEnd = this.dateEndCostControl.value;

      if(!dateStart || !dateEnd) return { isValid: false };

      if(dateStart === dateEnd) return { isValid: false };

      if (new Date(dateStart) > new Date(dateEnd)) return { isValid: false };

      return null;
    };
  }

  // Function OnSave

  onSaveFlightConfig(): void {
    this.flightBaseConfig.markAllAsTouched();
    if (this.flightBaseConfig.valid && this.scales.length > 0 && this.costs.length > 0) {
      const flightConfig: Flight = {
        id: 0,
        airline: this.airlines.find(airline => airline.id === this.airlineIdControl.value)?.name || '',
        airlineId: this.airlineIdControl.value,
        airportOrigin: this.scales[0].airportOrigin,
        airportOriginId: this.scales[0].airportOriginId,
        airportDestination: this.scales[this.scales.length - 1].airportDestination,
        airportDestinationId: this.scales[this.scales.length - 1].airportDestinationId,
        dateStart: this.scales[0].dateStart,
        dateEnd: this.scales[this.scales.length - 1].dateEnd,
        aircraftType: this.aircrafts.find(aircraft => aircraft.value === this.aircraftTypeIdControl.value)?.text || '',
        aircraftTypeId: Number.parseInt(this.aircraftTypeIdControl.value),
        scales: this.scales,
        costs: this.costs
      }

      this.flightService.save(flightConfig)
        .subscribe(response => {
          console.log(response);
        });

      this.flightBaseConfig.reset();
      this.scales = [];
      this.costs = [];
      this.location.back();
    }
  }

  onSaveScale(): void {
    this.scalesForm.markAllAsTouched();
    if (this.scalesForm.valid) {

      const scale: ScaleFlight = {
        id: this.scales.length + 1,
        airportOrigin: this.airports.find(airport => airport.value === this.airportIdOriginControl.value)?.text || '',
        airportOriginId: Number.parseInt(this.airports.find(airport => airport.value === this.airportIdOriginControl.value)?.value || ''),
        airportDestination: this.airports.find(airport => airport.value === this.airportIdDestinationControl.value)?.text || '',
        airportDestinationId: Number.parseInt(this.airportIdDestinationControl.value) || 0,
        dateStart: this.dateStartControl.value,
        dateEnd: this.dateEndControl.value,
      }
      console.log(scale);

      this.scales.push(scale);
      this.onTogleDialogScale();
      this.scalesForm.reset();
    }
  }

  onSaveCost(): void {
    this.costsForm.markAllAsTouched();
    if (this.costsForm.valid) {
      const cost: CostFlight = {
        id: this.costs.length + 1,
        dateStart: this.dateStartCostControl.value,
        dateEnd: this.dateEndCostControl.value,
        aircraftType: this.aircrafts.find(aircraft => aircraft.value === this.aircraftIdControl.value)?.text || '',
        aircraftTypeId: Number.parseInt(this.aircraftIdControl.value) || 0,
        cost: this.costControl.value
      }
      console.log(cost);
      this.costs.push(cost);
      this.onTogleDialogCost();
      this.costsForm.reset();
    }
  }
}
