import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { Flight } from '../interfaces/flight.interface';

@Injectable({providedIn: 'root'})
export class FlightService {
  constructor(private http: HttpClient) { }

  URL_BASE: string = 'http://localhost:3000/flights';

  getAll(): Observable<Flight[]> {
    return this.http.get<Flight[]>(this.URL_BASE);
  }

  searchFlight(origin: string, destination: string, dateStart: string): Observable<Flight[]> {
    return this.http.get<Flight[]>(`${this.URL_BASE}?airportOrigin_like=${origin}&airportDestination_like=${destination}&dateStart_like=${dateStart}`);
  }

  getById(id: number): Observable<Flight> {
    return this.http.get<Flight>(`${this.URL_BASE}/${id}`);
  }

  save(flight: Flight): Observable<Flight> {
    return this.http.post<Flight>(this.URL_BASE, flight);
  }

  update(flight: Flight): Observable<Flight> {
    return this.http.put<Flight>(`${this.URL_BASE}/${flight.id}`, flight);
  }

  delete(id: number): Observable<Flight> {
    return this.http.delete<Flight>(`${this.URL_BASE}/${id}`);
  }
}
